/**************************************************************************
 ***          Programme gestion Focalisation                            ***
 ***                                                                    ***
 **************************************************************************
 ***          Programmeur: Laurent Bernasconi                           ***
 ***          Version : 1.0-2002                                        ***
 ***          Debut: 14/10/2002     Fin:                                ***
 **************************************************************************/

#include <16F873.H>

#fuses  HS,NOWDT,NOPROTECT,NOPUT,NOBROWNOUT

#use delay(clock=20000000)
#use rs232(baud=9600, xmit=PIN_C6, rcv=PIN_C7)

#use fast_io(A)
#use fast_io(B)
#use fast_io(C)

#BIT    RCIF = 0x0C.5

#byte PORTA = 5
#byte PORTB = 6
#byte PORTC = 7


/****************************************************************
*                  Definition des Output                        *
****************************************************************/


#define ONEPHASE        PIN_A0  // Sequence 1 ou 2 phases aliment
#define HALFSTEP       	PIN_A1  // Mode pas ou demi pas
#define STEPINPUT       PIN_A2  // Horloge d'avancement moteur
#define DIRECTION 	PIN_A3  // Selection du sens de rotation moteur
#define OUTENABLE  	PIN_A4  // Validation de l'alimentation moteur


/****************************************************************
*                  Definition des Input                         *
****************************************************************/

#define CAPTEURINDUC    PIN_B3  // Ligne de lecture de l'etat du capteur inductif

/****************************************************************
*                  Definition IEC                               *
****************************************************************/

// #define EEPROM_SDA  PIN_C4
// #define EEPROM_SCL  PIN_C3



/***************************************************************************
 ******         Definition des variables        ****************************
 ***************************************************************************/

/*
 * Variables globales
 *
 */
 
long 	CompteurPas; //Compteur de pas entre deux dtections
long	ValCible;
char	Ordre;
int	i;
int	const Offset=40; //Valeur de ratrapage de jeu
int	Vitesse=5; // Valeur de dplacement du moteur en 1=2ms.
int	ValMen;
int	ValCmp1;
int	ValCmp2;
int	ValCmp3;
int	ValCmp4;
int	ValCmp5;
long	CompteurPasInter;
long	ValConv;
int	OffsetMoteur;

/*************************************************************************************/
/********************************      Fonctions      ********************************/
/*************************************************************************************/


/****************************************************************
*       Function Name:  boot                                    *
*       Return Value:   void                                    *
*       Parametres:                                             *
*       Description:    Renvoie vers le code du booloader       *
****************************************************************/

#ORG 0x0F00,0x0FFF
RESERVE()
{
}

/****************************************************************
*       Function Name:  LongToStrPerso                               *
*       Return Value:   void                                    *
*       Parametres:                                             *
*       Description:                                            *
****************************************************************/

void LongToStrPerso()
{							//10523
	ValCmp1=ValConv/10000;				//1
	CompteurPasInter=ValConv % 10000;		//523
	ValCmp2=(CompteurPasInter)/1000;		//0
	CompteurPasInter=CompteurPasInter % 1000;	//523
	ValCmp3=(CompteurPasInter)/100;			//5
	CompteurPasInter=CompteurPasInter % 100;	//23
	ValCmp4=(CompteurPasInter)/10;			//2
	CompteurPasInter=CompteurPasInter % 10;		//3
	ValCmp5=CompteurPasInter;			//3
	ValCmp1=ValCmp1+48;
	ValCmp2=ValCmp2+48;
	ValCmp3=ValCmp3+48;
	ValCmp4=ValCmp4+48;
	ValCmp5=ValCmp5+48;
	
	printf("%c%c%c%c%c",ValCmp1,ValCmp2,ValCmp3,ValCmp4,ValCmp5); // Fin de commande Retourne le compteur
	printf(" ");	
}

/****************************************************************
*       Function Name:  ConvertCharToInt()                      *
*       Return Value:   void                                    *
*       Parametres:                                             *
*       Description:                                            *
****************************************************************/

void ConvertCharToInt()
{	
	if (Ordre==48)
		ValMen=0;		
	else if (Ordre==49)
		ValMen=1;
	else if (Ordre==50)
		ValMen=2;
	else if (Ordre==51)
		ValMen=3;
	else if (Ordre==52)
		ValMen=4;
	else if (Ordre==53)
		ValMen=5;
	else if (Ordre==54)
		ValMen=6;
	else if (Ordre==55)
		ValMen=7;
	else if (Ordre==56)
		ValMen=8;
	else if (Ordre==57)
		ValMen=9;
	else
		ValMen=255;
}


/****************************************************************
*       Function Name:  AvanceUnPas                             *
*       Return Value:                                           *
*       Parametres:     adresse de la variable                  *
*       Description:    Avance le moteur d'un pas               *
****************************************************************/

void AvanceUnPas()

{
	OUTPUT_HIGH(STEPINPUT);	//pin4
	delay_ms(Vitesse);
	OUTPUT_LOW(STEPINPUT);	//pin4
	delay_ms(Vitesse);
}


/****************************************************************
*       Function Name:  SurDetecteur                            *
*       Return Value:   CompteurPas  long                       *
*       Parametres:     adresse de la variable                  *
*       Description:    Replace la ocalisation au niveau du     *
*                       capteur inductif                        *
****************************************************************/

void SurDetecteur()

{
	OUTPUT_LOW(DIRECTION);	//5 sens de base= 0
	OUTPUT_LOW(OUTENABLE);	//6 Valide l'alimentation du moteur
	delay_ms(10);
	while (!input(CAPTEURINDUC))
	{
		AvanceUnPas();
        }
        
        delay_ms(500);
        
        OUTPUT_HIGH(DIRECTION);	//5 sens de base= 1
        for (i=1;i<=Offset;++i)
        	AvanceUnPas();
        
        delay_ms(500);
        OUTPUT_HIGH(OUTENABLE);	//6 Coupe Alim. Moteur
        compteurPas=0;
        printf ("OK! ");	// Fin de commande... retourne OK!

}

/****************************************************************
*       Function Name:  PointeCible                             *
*       Return Value:   CompteurPas  long                       *
*       Parametres:     adresse de la variable                  *
*       Description:    Dplace la foc. sur la valeur choisie   *
****************************************************************/

void PointeCible()

{
	if (ValCible>compteurPas)
	{
		OUTPUT_HIGH(DIRECTION);	//5 sens de base= 1
		OUTPUT_LOW(OUTENABLE);	//6 Valide l'alimentation du moteur
		delay_ms(10);
Suite1:		AvanceUnPas();
		compteurPas=compteurPas+1;
		
		if (compteurPas<ValCible)
			Goto Suite1;
		
		delay_ms(10);
		OUTPUT_HIGH(OUTENABLE);	//6 Coupe Alim. Moteur
	}
	
	if (ValCible<compteurPas)
	{	
		OUTPUT_LOW(DIRECTION);	//5 sens de base= 0
		OUTPUT_LOW(OUTENABLE);	//6 Valide l'alimentation du moteur	
		delay_ms(10);
Suite2:		AvanceUnPas();
		compteurPas=compteurPas-1;
		
		if (compteurPas>ValCible)
			Goto Suite2;
		
		for (i=1;i<=Offset;++i)
        		AvanceUnPas();
        	
        	OUTPUT_HIGH(DIRECTION);	//5 sens de base= 1
        	delay_ms(500);
        	for (i=1;i<=Offset;++i)
        		AvanceUnPas();
        	
		delay_ms(10);
		OUTPUT_HIGH(OUTENABLE);	//6 Coupe Alim. Moteur
	}
	ValConv=CompteurPas;
	LongToStrPerso();	// retourne la valeur du compteur
}



/*************************************************************************************/
/******************************** Programme principal ********************************/
/*************************************************************************************/

main()
{
	

	SETUP_ADC_PORTS(NO_ANALOGS);

        SET_TRIS_A(0x00);           // Pins du PORTA en output
        SET_TRIS_B(0xFF);           // Pins du PORTB en input
        SET_TRIS_C(0xFF);           // PORTC en input

	PORT_B_PULLUPS(TRUE);
	
	OUTPUT_HIGH(ONEPHASE);	//2
	OUTPUT_LOW(HALFSTEP);	//3 sequence une phase aliment
	
	OUTPUT_LOW(STEPINPUT);  //4
	OUTPUT_LOW(DIRECTION);	//5 sens de base= 0
	OUTPUT_LOW(OUTENABLE);	//6 Valide l'alimentation du moteur

        //init_I2C();
 	//Init_Valeur_EEPROM();

 	SurDetecteur();		// Inicialisation de la foc. sur le dtecteur
 	
 	do
        {
FinFonction:    Ordre=getc();
        	
        	if (Ordre==0x6E)
        	{
        		ValConv=CompteurPas;	// commande n. Retourne la valeur du compteur
        		LongToStrPerso();	
		}
		
		else if (Ordre==0x2B)
		{
			Ordre=getc();		// commande +. Avance le moteur d'un offset positif compris entre 1 et 99
			ConvertCharToInt();
			if (Valmen==255)
			{
				Printf ("Erreur! ");
				goto FinFonction;
			}
			OffsetMoteur=0;
			if (Valmen>0)
			{
				for (i=1;i<=Valmen;++i)
                        		OffsetMoteur=OffsetMoteur+10;
                        }
                        
                        Ordre=getc();
			ConvertCharToInt();
			if (Valmen==255)
			{
				Printf ("Erreur! ");
				goto FinFonction;
			}
			OffsetMoteur=OffsetMoteur+Valmen;
			
			ValCible=CompteurPas+OffsetMoteur;
			PointeCible();
		}
		
		else if (Ordre==0x2D)
		{
			Ordre=getc();		// commande -. Avance le moteur d'un offset ngatif compris entre 1 et 99
			ConvertCharToInt();
			if (Valmen==255)
			{
				Printf ("Erreur! ");
				goto FinFonction;
			}
			OffsetMoteur=0;
			if (Valmen>0)
			{
				for (i=1;i<=Valmen;++i)
                        		OffsetMoteur=OffsetMoteur+10;
                        }
                        
                        Ordre=getc();
			ConvertCharToInt();
			if (Valmen==255)
			{
				Printf ("Erreur! ");
				goto FinFonction;
			}
			OffsetMoteur=OffsetMoteur+Valmen;
			
			ValCible=CompteurPas-OffsetMoteur;
			PointeCible();
		}
			
		else if (Ordre==0x64)
		{
			Ordre=getc(); // commande d = Recoit la valeur Cible puis dplace la foc 
			ConvertCharToInt();
			if (Valmen==255)
			{
				Printf ("Erreur! ");
				goto FinFonction;
			}
			ValCible=0;
			if (Valmen>0)
			{
				for (i=1;i<=Valmen;++i)
                        		ValCible=ValCible+10000;
                        }
                        
                        Ordre=getc(); //
			ConvertCharToInt();
			if (Valmen==255)
			{
				Printf ("Erreur! ");
				goto FinFonction;
			}
			if (Valmen>0)
			{
			for (i=1;i<=Valmen;++i)
                        	ValCible=ValCible+1000;
                        }
                        
                        Ordre=getc(); //
			ConvertCharToInt();
			if (Valmen==255)
			{
				Printf ("Erreur! ");
				goto FinFonction;
			}
			if (Valmen>0)
			{
				for (i=1;i<=Valmen;++i)
                        		ValCible=ValCible+100;
                        }
                        
                        Ordre=getc(); //
			ConvertCharToInt();
			if (Valmen==255)
			{
				Printf ("Erreur! ");
				goto FinFonction;
			}
			if (Valmen>0)
			{
				for (i=1;i<=Valmen;++i)
                        		ValCible=ValCible+10;
                        }
                        
                        Ordre=getc(); //
			ConvertCharToInt();
			if (Valmen==255)
			{
				Printf ("Erreur! ");
				goto FinFonction;
			}
                        ValCible=ValCible+ValMen;
		
			// ValConv=ValCible;
			// LongToStrPerso();	// retourne la valeur de la cible
		
			PointeCible();
		}

        	else if (Ordre==0x52)
			SurDetecteur(); // commande R = Rinisialise le moteur sur le capteur inductif
			
		else if (Ordre==0x56)
			printf("V1.0"); // Commande V = Retourne la version du code
	}
	while (true);
}
